#########################################################################
# Hermite Polynomial
#########################################################################
fH <- function( x, j ){
 m = t( matrix(0:floor(j/2), floor(j/2)+1, length(x)) )
 x = matrix(x, length(x), floor(j/2)+1)
 H = factorial(j) * (-1)^m / factorial(m) / factorial(j-2*m) * (2*x)^(j-2*m)
 H = cbind(H,0)
 H = rowSums( H[,1:(2+floor(j/2))] ) 
 return( H )
}

#########################################################################
# Hermite Function
#########################################################################
fpsi <- function( x, j ){
 return( (2^j * factorial(j) * 1.772454)^(-0.5) * exp(-x^2/2) * fH( x, j ) )
}

#########################################################################
# Hermite Function - 1st Derivative
#########################################################################
fpsi_prime <- function( x, j ){
 return( x*fpsi( x, j ) - (2*(j+1))^0.5 * fpsi( x, j+1) ) 
}

#########################################################################
# For Checking
#########################################################################
# j=1
# x = -100:100/20
# plot( x, fH( x, j ), ylim=c(-50,50), xlim=c(-2.5,2.5) )
# plot( x, fpsi( x, j ), ylim=c(-0.8,0.8), xlim=c(-5,5) )
# plot( x, fpsi_prime( x, j ), ylim=c(-0.8,0.8), xlim=c(-5,5) )